import { json, csv, select, zoom } from 'd3';
import topojson from 'topojson-client';
import { map } from './map';

const worldAtlasURL =
  'https://unpkg.com/visionscarto-world-atlas@0.1.0/world/110m.json';

const citiesURL =
  'https://gist.githubusercontent.com/curran/a59ef43debb9fcfd38858d0be4f3b087/raw/a56bdbdb758eebf6a387d47e4d428258e5cb2abd/worldcitiesReduced.csv';

export const main = (container, { state, setState }) => {
  // const width = ;
  // const height = ;

  // For this once we use a fixed size
  // then use the `viewBox` trick to scale the SVG.
  const width = 960;
  const height = 500;

  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', '100%')
    .attr('height', '100%')
    .attr('viewBox', `0 0 ${width} ${height}`)
    .attr('preserveAspectRatio', 'xMidYMid meet')
    .style('background-color', '#000000');

  const zoomG = svg
    .selectAll('g.zoomable')
    .data([null])
    .join('g')
    .attr('class', 'zoomable');

  // enable zooming with d3-zoom
  svg.call(
    zoom().on('zoom', (event) => {
      zoomG.attr('transform', event.transform);
    }),
  );

  // state.data could be:
  // * undefined
  // * 'LOADING'
  // * An array of objects
  const { countries, cities, land, countryBorders } = state;

  if (countries === undefined) {
    setState((state) => ({
      ...state,
      countries: 'LOADING',
    }));
    Promise.all([
      json(worldAtlasURL),
      csv(citiesURL, (d) => {
        d.lat = +d.lat;
        d.lng = +d.lng;
        d.population = +d.population;
        return d;
      }),
    ]).then(([topoJSONData, cities]) => {
      const countries = topojson.feature(
        topoJSONData,
        'countries',
      );
      const land = topojson.feature(topoJSONData, 'land');
      const countryBorders = topojson.mesh(
        topoJSONData,
        topoJSONData.objects.countries,
        (a, b) => a !== b,
      );
      setState((state) => ({
        ...state,
        countries,
        land,
        countryBorders,
        cities,
      }));
    });
    return;
  }
  if (countries === 'LOADING') {
    return;
  }

  zoomG.call(map, {
    countries,
    cities,
    width,
    height,
    land,
    countryBorders,
  });
};
